/*
 * Copyright:      Jan Homann
 * Author   :      Jan Homann 
 * Version  :      1.0 
 * Date     :	   24.02.2016
 * Source   :      none
 */



#include <avr/io.h>
#include "spi.h"
#include "AD7758.h"

/* contains read out values */
uint8_t Buffer[3];



uint8_t ADE7758_read_uint8(uint8_t reg)
{
	Buffer[0] = 0;
	
	spi_enable_chip();
	spi_disable_chip(); // start transmission condition  
	
	spi_master_transmit(reg); // register address
	spi_master_transmit(0x00); // dummy byte	
	
	spi_enable_chip(); // stop transmission condition
	
	Buffer[0] = spi_receive(); // first byte		
	
	return Buffer[0];
}

int8_t ADE7758_read_int8(uint8_t reg)
{
		int8_t temp = 0;
		
		spi_enable_chip();
		spi_disable_chip(); // start transmission condition
		
		spi_master_transmit(reg); // register address
		spi_master_transmit(0x00); // dummy byte
		
		spi_enable_chip(); // stop transmission condition
		
		temp = (int8_t)spi_receive(); // first byte
		
		return (int8_t) temp;
}

uint16_t ADE7758_read_uint16(uint8_t reg)
{
	Buffer[0] = 0; // default value
	Buffer[1] = 0; // default value
	
	spi_enable_chip();
	spi_disable_chip(); // start transmission condition
	
	spi_master_transmit(reg); // register address
	spi_master_transmit(0x00); // dummy byte
	
	spi_enable_chip(); // stop transmission condition
	
	Buffer[0] = spi_receive(); // first byte
	Buffer[1] = spi_receive(); // secound byte
	
					  /* first byte		   secound byte */
	uint16_t result = (((8<<Buffer[0])) | Buffer[1]);
	
	return result;
}

signed long ADE7758_read_int32(uint8_t reg)
{
	int8_t byte_1 = 0;
	int8_t byte_2 = 0;
	int8_t byte_3 = 0;
	
	spi_enable_chip();
	spi_disable_chip(); // start transmission condition
	
	spi_master_transmit(reg);  // register address
	spi_master_transmit(0x00); // dummy byte
	byte_1 = (int8_t)spi_receive(); // first byte
	
	spi_master_transmit(0x00);
	byte_2 = (int8_t)spi_receive(); // second byte
	
	spi_master_transmit(0x00);
	byte_3 = (int8_t)spi_receive(); // third byte
	
	spi_enable_chip(); // stop transmission condition	
							/*     first byte			second byte	    third byte */
	signed long result = ((16 << byte_1) |  (8 << byte_2) | byte_3);
	
	return result;
}

void ADE7758_write_uint8(uint8_t reg, uint8_t data)
{
	spi_enable_chip();
	spi_disable_chip();

	spi_master_transmit(reg | 0x80); // 0x80 for write command
	spi_master_transmit(data); // data transmission

	spi_enable_chip();
}

int8_t ADE7758_readTEMP_OFFSET(void)
{
	return (int8_t)ADE7758_read_int8(REG_TEMP);
}

int8_t ADE7758_readTEMP(void)
{
	int8_t temperature = 0;
	
	temperature	 = ADE7758_read_int8(REG_TEMP); // raw data
	temperature -= 40; // offset for this sensor
	temperature *= 3; // 3C/Bit
	//temperature += 21; // ambient temperature
	
	return temperature;
}

signed long ADE7758_get_U(uint8_t phase)
{
	signed long voltage = 0;

	switch (phase)
	{
		case 1	: {voltage = ADE7758_read_int32(REG_AVRMS);	}break; // L1			 [Voltage]
		case 2	: {voltage = ADE7758_read_int32(REG_BVRMS);	}break; // L2			 [Voltage]
		case 3	: {voltage = ADE7758_read_int32(REG_CVRMS);	}break; // L3			 [Voltage]
		default	: {voltage = ADE7758_read_int32(REG_AVRMS);	}break; // default = L1  [Voltage]
	}

	return voltage;
}