/*
 * Copyright:      Jan Homann
 * Author   :      Jan Homann 
 * Version  :      1.0 
 * Date     :	   20.10.2015
 * Source   :      -Datasheet (MEGA32)
 */


#include <avr/interrupt.h>

#define MOSI_DDR	DDRB
#define MOSI_PORT	PORTB
#define MOSI_BIT	2

#define MISO_DDR	DDRB
#define MISO_PIN	PINB
#define MISO_PORT	PORTB
#define MISO_BIT	3

#define SCK_DDR		DDRB
#define SCK_PORT	PORTB
#define SCK_BIT		1

/* software slave select signal */
#define SS_DDR		DDRB
#define SS_PORT		PORTB
#define SS_BIT		6

/* hardware slave select signal */
#define SS_HA_DDR	DDRB
#define SS_HA_PORT	PORTB
#define SS_HA_BIT	0

/* Prescalers */
#define PRESCALER_2		2
#define PRESCALER_4		4
#define PRESCALER_8		8
#define PRESCALER_16	16
#define PRESCALER_32	32
#define PRESCALER_64	64	
#define PRESCALER_128	128






/* Only for MEGA32 

Pin		Direction, Master SPI		Direction, Slave SPI

MOSI	User Defined				Input
MISO	Input						User Defined
SCK		User						Defined Input
SS		User						Defined Input

*/




/***Prototype functions***/
 
 
 
 
/* init spi unit @ master
* parameter : prescaler : 
*							fclk/2	= 2  (prescaler)
*							fclk/8	= 8  (prescaler)
*							fclk/32	= 32 (prescaler)
*							fclk/64 = 64 (prescaler)
*							
* returns	: none	           
*/
void spi_master_init(uint8_t prescaler, uint8_t clock_option);


/* init spi unit @ slave 
* parameter : prescaler : 
*							fclk/4	=  4   (prescaler)
*							fclk/16	=  16  (prescaler)
*							fclk/64	=  64  (prescaler)
*							fclk/128 = 128 (prescaler)
* returns	 : none							               
*/
void spi_slave_init(uint8_t prescaler);


/* send data via. spi 
* parameter : cData : 8 - Bit Data    
* returns	: none	             
*/
void spi_master_transmit(uint8_t cData);


/* receive data via. spi
* parameter : none
* returns   : 8 Bit Data from Slave 
*/
uint8_t spi_receive(void);


/* chip select active
* parameter : none
* returns   : none
*/
void spi_enable_chip(void);


/* chip select deactive
* parameter : none
* returns   : none
*/
void spi_disable_chip(void);


extern volatile uint8_t data[8];